local addon = {
	name = "VotansFishFillet",
	defaults =
	{
		stats = { fishes = 0, perfectRoe = 0 }
	},
}

local stats

------------ Filleting a fish ----------------
local useCallProtectedFunction = IsProtectedFunction("UseItem")
local function IsItemFish(bagId, slotIndex)
	if bagId ~= BAG_BACKPACK then return false end

	local itemType = GetItemType(bagId, slotIndex)
	if ITEMTYPE_FISH == itemType then
		local usable, onlyFromActionSlot = IsItemUsable(bagId, slotIndex)
		local canInteractWithItem = CanInteractWithItem(bagId, slotIndex)
		return usable and not onlyFromActionSlot and canInteractWithItem
	end
	return false
end

local FiletFish
local count = 0
local roeCount = 0
local itemLink
local slotIndex
local updateExpected = false

local function CountPerfectRoe()
	local git = GetItemLink
	local gii = GetItemInfo
	local zpi = ZO_LinkHandler_ParseLink
	local sum = 0
	local _, count
	local bagId = BAG_BACKPACK
	local bagSlots = GetBagSize(BAG_BACKPACK) -1
	for index = 0, bagSlots do
		local itemId = select(4, zpi(git(bagId, index)))
		if itemId == "64222" then
			_, count = gii(bagId, index)
			sum = sum + count
		end
	end
	return sum
end

local function AbortAction(...)
	EVENT_MANAGER:UnregisterForEvent(addon.name, EVENT_INVENTORY_FULL_UPDATE)
	EVENT_MANAGER:UnregisterForEvent(addon.name, EVENT_INVENTORY_SINGLE_SLOT_UPDATE)
	-- Count "Perfect Roe" afterwards and add diff to stats
	local newCount = CountPerfectRoe()
	stats.perfectRoe = stats.perfectRoe +(newCount - roeCount)
	roeCount = newCount
	ShowMouse()
end

local function InventoryStateChange(oldState, newState)
	if newState == SCENE_HIDING then
		count = 0
		AbortAction()
		INVENTORY_FRAGMENT:UnregisterCallback("StateChange", InventoryStateChange)
	end
end

local function EndAction(...)
	AbortAction()
	INVENTORY_FRAGMENT:UnregisterCallback("StateChange", InventoryStateChange)
end

local function SlotUpdate(eventCode, bagId, slotId, isNew, itemSoundCategory, updateReason)
	if bagId ~= BAG_BACKPACK or updateReason ~= INVENTORY_UPDATE_REASON_DEFAULT then return end

	if isNew and updateExpected then
		local itemId = select(4, ZO_LinkHandler_ParseLink(GetItemLink(bagId, slotId)))
		if itemId == "33753" then
			count = count - 1
			stats.fishes = stats.fishes + 1
			updateExpected = false
			if count > 0 then
				-- Not zo_callLater: if there is a delay already, it will not registered twice
				EVENT_MANAGER:RegisterForUpdate(addon.name, 40, FiletFish)
			else
				EndAction()
			end
		end
	end
end

-- After reorg of the inventory, e.g. after new entry "fish", find the stack again
local function SlotFullUpdate(eventCode, bagId, ...)
	if bagId ~= BAG_BACKPACK then return end

	if IsItemFish(bagId, slotIndex) then
		SlotUpdate(eventCode, BAG_BACKPACK, slotIndex, true, nil, INVENTORY_UPDATE_REASON_DEFAULT)
		return
	end

	local bagSlots = GetBagSize(bagId) -1
	local found = false
	local git = GetItemLink
	local gii = GetItemInfo
	for index = 0, bagSlots do
		if itemLink == git(bagId, index) then
			found = true
			slotIndex = index
			local _, stack = gii(bagId, slotIndex)
			if stack ==(count - 1) then break end
		end
	end
	if found then
		SlotUpdate(eventCode, BAG_BACKPACK, slotIndex, true, nil, INVENTORY_UPDATE_REASON_DEFAULT)
	else
		EndAction()
	end
end

FiletFish = function()
	EVENT_MANAGER:UnregisterForUpdate(addon.name)
	if not CheckInventorySpaceSilently(2) then
		EndAction()
		ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, SI_INVENTORY_ERROR_INVENTORY_FULL)
		return false
	end
	if IsItemFish(BAG_BACKPACK, slotIndex) then
		EVENT_MANAGER:RegisterForEvent(addon.name, EVENT_INVENTORY_FULL_UPDATE, SlotFullUpdate)
		EVENT_MANAGER:RegisterForEvent(addon.name, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, SlotUpdate)

		updateExpected = true
		if useCallProtectedFunction then
			if not CallSecureProtected("UseItem", BAG_BACKPACK, slotIndex) then
				EndAction()
				PlaySound(SOUNDS.NEGATIVE_CLICK)
				return false
			end
		else
			UseItem(BAG_BACKPACK, slotIndex)
		end
		-- inventorySlot is unvalid afterwards, because the position could have changed due to the new entry "fish"
	else
		EndAction()
	end
	return false
end

local function UseInventorySlot(inventorySlot, slotActions)
	local bagId, _
	bagId, slotIndex = ZO_Inventory_GetBagAndIndex(inventorySlot)

	if IsItemFish(bagId, slotIndex) then
		_, count = GetItemInfo(bagId, slotIndex)
		-- Count "Perfect Roe" before
		roeCount = CountPerfectRoe()
		if count > 0 then
			itemLink = GetItemLink(bagId, slotIndex)
			INVENTORY_FRAGMENT:RegisterCallback("StateChange", InventoryStateChange)
			EVENT_MANAGER:RegisterForUpdate(addon.name, 40, FiletFish)
			slotActions:Clear()
			ClearCursor()
			ClearMenu()
			HideMouse()
			return true
		else
			itemLink = ""
			EndAction()
		end
	end
	return false
end

local function IsPlayerAlive()
	return not IsUnitDead("player")
end

local function AddSlotAction(self, actionStringId, actionCallback, actionType)
	local actionName = GetString(actionStringId)
	local options = { visibleWhenDead = false }

	table.insert(self.m_slotActions, { actionName, actionCallback, actionType, IsPlayerAlive, options })
	self.m_hasActions = true

	if self.m_contextMenuMode and IsPlayerAlive() then
		AddCustomMenuItem(actionName, actionCallback)
		self.m_numContextMenuActions = self.m_numContextMenuActions + 1
	end
end

local function AddFiletAll(inventorySlot, slotActions)
	local bagId, slotIndex = ZO_Inventory_GetBagAndIndex(inventorySlot)
	if IsItemFish(bagId, slotIndex) and CheckInventorySpaceSilently(2) and BACKPACK_MENU_BAR_LAYOUT_FRAGMENT:GetState() == SCENE_SHOWN then
		AddSlotAction(slotActions, SI_VOTANS_FILET_FISH_ALL, function(...) return UseInventorySlot(inventorySlot, slotActions) end, "primary")
	end
	return false
end

ZO_PreHook("ZO_InventorySlot_DiscoverSlotActionsFromActionList", AddFiletAll)

------------ End Filleting a fish ----------------

---------------- tooltip stats -------------------

local function ModifyTooltip(tooltip, itemLink)
	if GetItemLinkItemType(itemLink) ~= ITEMTYPE_FISH then return end
	if stats.fishes > 0 then
		tooltip:AddVerticalPadding(10)
		tooltip:AddLine(zo_strjoin(nil, addon.perfectRoeName, ": ", stats.perfectRoe, "/", stats.fishes, " (", math.floor(stats.perfectRoe * 10000 / stats.fishes) / 100, "%)"))
	end
end

local function TooltipHook(tooltipControl, method, linkFunc, equipped)
	local origMethod = tooltipControl[method]

	tooltipControl[method] = function(self, ...)
		origMethod(self, ...)
		ModifyTooltip(self, linkFunc(...))
	end
end

local function ReturnItemLink(itemLink)
	return itemLink
end

------------- end tooltip stats ------------------

-- Add stats for perfect roe
-- [64221] = "Psijik-Ambrosia^ns",
-- [64222] = "perfekter Rogen^ms",
-- [64223] = "Rezept: Psijik-Ambrosia^n:ns",

local function OnAddonLoaded(event, name)
	if name ~= addon.name then return end
	EVENT_MANAGER:UnregisterForEvent(addon.name, EVENT_ADD_ON_LOADED)

	addon.settings = ZO_SavedVars:NewAccountWide("VotanFishFillet_Data", 1, nil, addon.defaults)
	stats = addon.settings.stats

	addon.perfectRoeName = zo_strformat(SI_TOOLTIP_ITEM_NAME, GetItemLinkName(string.format("|H0:item:%i:20:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|htemp|h", 64222)))

	TooltipHook(ItemTooltip, "SetBagItem", GetItemLink)
	TooltipHook(ItemTooltip, "SetTradeItem", GetTradeItemLink)
	TooltipHook(ItemTooltip, "SetBuybackItem", GetBuybackItemLink)
	TooltipHook(ItemTooltip, "SetStoreItem", GetStoreItemLink)
	TooltipHook(ItemTooltip, "SetAttachedMailItem", GetAttachedItemLink)
	TooltipHook(ItemTooltip, "SetLootItem", GetLootItemLink)
	TooltipHook(ItemTooltip, "SetTradingHouseItem", GetTradingHouseSearchResultItemLink)
	TooltipHook(ItemTooltip, "SetTradingHouseListing", GetTradingHouseListingItemLink)
	TooltipHook(ItemTooltip, "SetLink", ReturnItemLink)

	TooltipHook(PopupTooltip, "SetLink", ReturnItemLink)

end

EVENT_MANAGER:RegisterForEvent(addon.name, EVENT_ADD_ON_LOADED, OnAddonLoaded)
